import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.awt.image.*;
import java.awt.geom.*;

/** This is a class for the eraser tool which is located at row 2 column 1 of the drawing icon.
 * It allows the user to erase what has been drawn. 
 *
 * It should work with all operating systems and hardware. 
 * There are no variances and no security constraints.
 *
 * @author TerpPaint
 * @version 2.0
 */
public class eraserTool implements ourTool {
    /** Holds the graphic that is actually used to erase.
     */    
    Graphics2D g2D;
    /** Holds the BufferedImage that we use to create the graphic to do the erasing.
     */    
    BufferedImage eraseImage;
    /** Holds the size of the eraser. It is initialized to 6 at the constructor.
     */    
    private int eraserSize;
    
   /** used to initialize the start of the x and y values
     */
    int prevX;
    
    /** used to initialize the start of the x and y values
     */
    int prevY;
    
    
    /** Creates an eraserTool and initializes its size to 6.
     * It takes in no parameters or null arguments.  It does not return anything.
     * There are no algorithms of any kind and no variances and OS dependencies.
     * There should not be any exceptions or security constraints.
     */    
    public eraserTool() {
        super();
        eraserSize = 6;
    }
    
    /** Allows the user to click on the canvas using the eraserTool.
     * It will get the BufferedImage from the canvas and prepare to draw
     * a graphic onto it. It has 2 parameters. The MouseEvent holds the button that
     * was clicked on (either left or right) and gets the coordinate
     * of where the mouse is clicked. The second parameter is the main_canvas which is the area
     * that can be erased.
     * @param mevt A MouseEvent object for showing which button in the mouse is clicked and the
     * coordinate of where the mouse is clicked.
     * @param theCanvas A main_canvas object which is the area that we are going to erase.
     */    
    public void clickAction(MouseEvent mevt, main_canvas theCanvas) {
        int x, y;
        
        eraseImage = theCanvas.getBufferedImage();
        theCanvas.setBufferedImage(eraseImage);
        g2D = eraseImage.createGraphics();
        x = theCanvas.canvasScale(mevt.getX());
        y = theCanvas.canvasScale(mevt.getY());
        if (SwingUtilities.isLeftMouseButton(mevt)) {
            g2D.setColor(theCanvas.right_color);
        } else {
            g2D.setColor(theCanvas.left_color);
        }
        g2D.fill(getEraserShape(x,y));
        
         prevX = x;
        prevY = y;
    }
    
    /** Allows the user to drag the mouse using the eraserTool, and therefore,
     * allows the user to erase until the mouse is released.
     * It has 2 parameters. The first one is a MouseEvent and it is to check which button
     * in the mouse is clicked and to get the coordinate of where the mouse is clicked.
     * The second parameter is the canvas which is the area that we are going to erase.
     * @param mevt A MouseEvent object for showing which button in the mouse is clicked and the
     * coordinate of where the mouse is clicked.
     * @param theCanvas A main_canvas object which is the area that we are going to erase.
     */    
    public void dragAction(MouseEvent mevt, main_canvas theCanvas) {
        /*int x, y;
        
        x = theCanvas.canvasScale(mevt.getX());
        y = theCanvas.canvasScale(mevt.getY());
        if (SwingUtilities.isLeftMouseButton(mevt)) {
            g2D.setColor(theCanvas.right_color);
        } else {
            g2D.setColor(theCanvas.left_color);
        }
        g2D.fill(getEraserShape(x,y));
        theCanvas.repaint();
        */
         int x, y;
        x = theCanvas.canvasScale(mevt.getX());
        y = theCanvas.canvasScale(mevt.getY());
        if (SwingUtilities.isLeftMouseButton(mevt)) {
            //pencilImage.setRGB(x,y,theCanvas.left_color.getRGB());
            g2D.setColor(theCanvas.right_color);
            g2D.setStroke(new BasicStroke(getEraserSize()));
            g2D.drawLine(prevX,prevY,x,y);
        } else {
            //pencilImage.setRGB(x,y,theCanvas.right_color.getRGB());
            g2D.setColor(theCanvas.left_color);
            g2D.setStroke(new BasicStroke(getEraserSize()));
            g2D.drawLine(prevX,prevY,x,y);
        }
       g2D.fill(getEraserShape(x,y));
        prevX = x;
        prevY = y;
        theCanvas.repaint();
    }
    
    /** Allows the user to release the mouse, and therefore stop the erasing process. 
     * It has 2 parameters. The first one is a MouseEvent and it
     * is to check which button in the mouse is clicked and to get the coordinate of where
     * the mouse is clicked. The second parameter is the canvas which is the area that we
     * are going to erase.
     * @param mevt A MouseEvent object for showing which button in the mouse is clicked and the
     * coordinate of where the mouse is clicked.
     * @param theCanvas A main_canvas object which is the area that we are going to erase.
     */    
    public void mouseReleaseAction(MouseEvent mevt, main_canvas theCanvas) {
     /*   int x, y;
        
        x = theCanvas.canvasScale(mevt.getX());
        y = theCanvas.canvasScale(mevt.getY());
        if (SwingUtilities.isLeftMouseButton(mevt)) {
            g2D.setColor(theCanvas.right_color);
        } else {
            g2D.setColor(theCanvas.left_color);
        }
        g2D.fill(getEraserShape(x,y));
        theCanvas.repaint(); 
       */ 
        
         int x, y;
        x = theCanvas.canvasScale(mevt.getX());
        y = theCanvas.canvasScale(mevt.getY());
        if (SwingUtilities.isLeftMouseButton(mevt)) {
            //pencilImage.setRGB(x,y,theCanvas.left_color.getRGB());
            g2D.setColor(theCanvas.right_color);
            g2D.setStroke(new BasicStroke(getEraserSize()));
            g2D.drawLine(prevX,prevY,x,y);
        } else {
            //pencilImage.setRGB(x,y,theCanvas.right_color.getRGB());
            g2D.setColor(theCanvas.left_color);
            g2D.setStroke(new BasicStroke(getEraserSize()));
            g2D.drawLine(prevX,prevY,x,y);
        }
         g2D.fill(getEraserShape(x,y));
        theCanvas.repaint();
        
        
    }
  
    /** This function returns the color set to the private data
     * member g2d.
     * @return Color representing g2d
     */    
    public Color getG2dColor(){ return g2D.getColor();}
       
    /** Returns the Shape of the eraserTool, as specified by the x and y values
     * passed in. It will return a Rectangle object which is the eraser. It has 
     * 2 int parameters which is for the coordinate of x and y.
     * @param x for x coordinate.
     * @param y for y coordinate.
     * @return Returns a Rectangular object which is the eraser.
     */    
    public Shape getEraserShape(int x, int y) {
        Rectangle eraser;
        Double theSize = new Double(eraserSize/2);
        eraser = new Rectangle(x-theSize.intValue(),y-theSize.intValue(),eraserSize,eraserSize);
        return eraser;
    }
    
    /** Sets the size of the eraser, as specified by the int parameter passed in.
     * @param newSize int value.
     */    
    public void setEraserSize(int newSize) { eraserSize = newSize; }
    
    /** Returns the size of the eraser as an int.
     * @return int eraserSize
     */    
    public int getEraserSize(){return eraserSize;};
}
